package Pugs::Grammar::Pod;
use strict;
use warnings;
use base qw(Pugs::Grammar::BaseCategory);

# XXX - this module is currently unused
# XXX - see Pugs::Grammar::BaseCategory ws()

# placeholder for the Pod grammar

BEGIN {

    # __PACKAGE__->recompile;
}


1;
